using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms; 

namespace IrregularForm
{
    public class BitmapRegion
    {
        public BitmapRegion()
        { }


        /// <summary> 
        /// Create and apply the region on the supplied control
        /// ֧λͼĿؼĿǰbuttonform
        /// </summary> 
        /// <param name="control">The Control object to apply the region toؼ</param> 
        /// <param name="bitmap">The Bitmap object to create the region fromλͼ</param> 
        public static void CreateControlRegion(Control control, Bitmap bitmap)
        {
            // Return if control and bitmap are null
            //жǷڿؼλͼ
            if (control == null || bitmap == null)
                return;

            // Set our control''s size to be the same as the bitmap
            //ÿؼСΪλͼС
            control.Width = bitmap.Width;
            control.Height = bitmap.Height;
            // Check if we are dealing with Form here 
            //ؼformʱ
            if (control is System.Windows.Forms.Form)
            {
                // Cast to a Form object
                //ǿתΪFORM
                Form form = (Form)control;
                // Set our form''s size to be a little larger that the  bitmap just 
                // in case the form''s border style is not set to none in the first place 
                //FORMı߽FormBorderStyleΪNONEʱӦFORMĴСóɱλͼСԴһ
                form.Width = control.Width;
                form.Height = control.Height;
                // No border 
                //ûб߽
                form.FormBorderStyle = FormBorderStyle.None;
                // Set bitmap as the background image 
                //λͼóɴ屳ͼƬ
                form.BackgroundImage = bitmap;
                // Calculate the graphics path based on the bitmap supplied 
                //λͼв͸ֵı߽
                GraphicsPath graphicsPath = CalculateControlGraphicsPath(bitmap);
                // Apply new region 
                //Ӧµ
                form.Region = new Region(graphicsPath);
            }
            // Check if we are dealing with Button here 
            //ؼbuttonʱ
            else if (control is System.Windows.Forms.Button)
            {
                // Cast to a button object 
                //ǿתΪ button
                Button button = (Button)control;
                // Do not show button text 
                //ʾbutton text
                button.Text = "";

                // Change cursor to hand when over button 
                //ı cursorstyle
                button.Cursor = Cursors.Hand;
                // Set background image of button 
                //buttonıͼƬ
                button.BackgroundImage = bitmap;

                // Calculate the graphics path based on the bitmap supplied 
                //λͼв͸ֵı߽
                GraphicsPath graphicsPath = CalculateControlGraphicsPath(bitmap);
                // Apply new region 
                //Ӧµ
                button.Region = new Region(graphicsPath);
            }
        }
        /// <summary> 
        /// Calculate the graphics path that representing the figure in the bitmap 
        /// excluding the transparent color which is the top left pixel. 
        /// //λͼв͸ֵı߽
        /// </summary> 
        /// <param name="bitmap">The Bitmap object to calculate our graphics path from</param> 
        /// <returns>Calculated graphics path</returns> 
        private static GraphicsPath CalculateControlGraphicsPath(Bitmap bitmap)
        {
            // Create GraphicsPath for our bitmap calculation 
            // GraphicsPath
            GraphicsPath graphicsPath = new GraphicsPath();
            // Use the top left pixel as our transparent color 
            //ʹϽǵһɫΪ͸ɫ
            Color colorTransparent = bitmap.GetPixel(0, 0);
            // This is to store the column value where an opaque pixel is first found. 
            // This value will determine where we start scanning for trailing opaque pixels.
            //һҵX
            int colOpaquePixel = 0;
            // Go through all rows (Y axis) 
            // ƫУY
            for (int row = 0; row < bitmap.Height; row++)
            {
                // Reset value 
                //
                colOpaquePixel = 0;
                // Go through all columns (X axis) 
                //ƫУX
                for (int col = 0; col < bitmap.Width; col++)
                {
                    // If this is an opaque pixel, mark it and search for anymore trailing behind 
                    //ǲҪ͸ĵǣȻƫ
                    if (bitmap.GetPixel(col, row) != colorTransparent)
                    {
                        // Opaque pixel found, mark current position
                        //¼ǰ
                        colOpaquePixel = col;
                        // Create another variable to set the current pixel position 
                        //±¼ǰ
                        int colNext = col;
                        // Starting from current found opaque pixel, search for anymore opaque pixels 
                        // trailing behind, until a transparent   pixel is found or minimum width is reached 
                        ///ҵĲ͸㿪ʼѰҲ͸,һֱҵﵽͼƬ 
                        for (colNext = colOpaquePixel; colNext < bitmap.Width; colNext++)
                            if (bitmap.GetPixel(colNext, row) == colorTransparent)
                                break;
                        // Form a rectangle for line of opaque   pixels found and add it to our graphics path 
                        //͸ӵgraphics path
                        graphicsPath.AddRectangle(new Rectangle(colOpaquePixel, row, colNext - colOpaquePixel, 1));
                        // No need to scan the line of opaque pixels just found 
                        col = colNext;
                    }
                }
            }
            // Return calculated graphics path 
            return graphicsPath;
        }
    } 

}
